﻿using DryIoc.ImTools;
using LiveCharts;
using LiveCharts.Wpf;
using Mysqlx.Crud;
using Newtonsoft.Json;
using Org.BouncyCastle.Utilities;
using StudentManager.Common;
using StudentManager.Data;
using StudentManager.Extensions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace StudentManager.Model
{
	class MenuBar : BindableBase
	{
		private string icon;
		/// <summary>
		/// Icon
		/// </summary>
		public string Icon
		{
			get { return icon; }
			set { icon = value; }
		}

		private string title;

		public string Title
		{
			get { return title; }
			set { title = value; }
		}


		private string nameSpace;

		public string NameSpace
		{
			get { return nameSpace; }
			set { nameSpace = value; }
		}


	}

	class Students : BindableBase
	{

		private ObservableCollection<StudentInfo> studentDatas = new ObservableCollection<StudentInfo>();
		public ObservableCollection<StudentInfo> StudentDatas { get { return studentDatas; } private set { studentDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<QuestionData> questionDatas = new ObservableCollection<QuestionData>();
		public ObservableCollection<QuestionData> QuestionDatas { get { return questionDatas; } private set { questionDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<QuestionData> homeworkTestData = new ObservableCollection<QuestionData>();
		public ObservableCollection<QuestionData> HomeworkTestData { get { return homeworkTestData; } private set { homeworkTestData = value; RaisePropertyChanged(); } }

		private ObservableCollection<QuestionData> preToRemove = new ObservableCollection<QuestionData>();
		private ObservableCollection<QuestionData> addQuestionDatas = new ObservableCollection<QuestionData>();
		public ObservableCollection<QuestionData> ADDQuestionDatas { get { return addQuestionDatas; } private set { addQuestionDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<DetailErrorInfo> errorSetDatas = new ObservableCollection<DetailErrorInfo>();
		public ObservableCollection<DetailErrorInfo> ErrorSetDatas { get { return errorSetDatas; } private set { errorSetDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<CursonQuestionsData> cQDatas = new ObservableCollection<CursonQuestionsData>();
		public ReadOnlyObservableCollection<CursonQuestionsData> CQDatas { get; private set; }


		private ObservableCollection<DetailHomeWorkSetInfo> homeWorkSet = new ObservableCollection<DetailHomeWorkSetInfo>();
		public ObservableCollection<DetailHomeWorkSetInfo> HomeWorkSet { get { return homeWorkSet; } private set { homeWorkSet = value; RaisePropertyChanged(); } }

		private ObservableCollection<DetailHomeWorkInfo> homeWork = new ObservableCollection<DetailHomeWorkInfo>();
		public ObservableCollection<DetailHomeWorkInfo> HomeWork { get { return homeWork; } private set { homeWork = value; RaisePropertyChanged(); } }

		private ObservableCollection<MenuBar> menuBars = new ObservableCollection<MenuBar>();
		public ReadOnlyObservableCollection<MenuBar> MenuBars { get; private set; }



		private int totalQuestions;
		public int TotalQuestions
		{
			get { return totalQuestions; }
			set { totalQuestions = value; }
		}


		private StudentInfo selectedStudent;
		public StudentInfo SelectedStudent
		{
			get { return selectedStudent; }
			set { selectedStudent = value; RaisePropertyChanged(); }
		}


		private DetailHomeWorkSetInfo selectedHomeWorkSet;
		public DetailHomeWorkSetInfo SelectedHomeWorkSet
		{
			get { return selectedHomeWorkSet; }
			set { selectedHomeWorkSet = value; RaisePropertyChanged(); }
		}

		private DetailHomeWorkInfo selectedHomeWork;
		public DetailHomeWorkInfo SelectedHomeWork
		{
			get { return selectedHomeWork; }
			set { selectedHomeWork = value; RaisePropertyChanged(); }
		}

		private StudentInfo studentInfo;

		public StudentInfo StudentInfo
		{
			get { return studentInfo; }
			set { studentInfo = value; RaisePropertyChanged(); }
		}

		private DetailErrorSetInfo errorSet;

		public DetailErrorSetInfo ErrorSet
		{
			get { return errorSet; }
			set { errorSet = value; RaisePropertyChanged(); }
		}

		private uint lesson;

		public uint Lesson
		{
			get { return lesson; }
			set { lesson = value; RaisePropertyChanged(); }
		}


		private bool isAddPublicQuestionsLib;

		public bool IsAddPublicQuestionsLib
		{
			get { return isAddPublicQuestionsLib; }
			set { isAddPublicQuestionsLib = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }

		}

		private bool isNeedErrorset;

		public bool IsNeedErrorset
		{
			get { return isNeedErrorset; }
			set { isNeedErrorset = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }
		}

		private bool isControlQuestionNum;

		public bool IsControlQuestionNum
		{
			get { return isControlQuestionNum; }
			set { isControlQuestionNum = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }
		}


		private uint publicLesson;

		public uint PublicLesson
		{
			get { return publicLesson; }
			set { publicLesson = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }
		}


		private int questionCount;

		public int QuestionCount
		{
			get { return questionCount; }
			set { questionCount = value; RaisePropertyChanged(); }
		}

		private QuestionData selectedQuestion = new QuestionData();

		public QuestionData SelectedQuestion
		{
			get { return selectedQuestion; }
			set { selectedQuestion = value; RaisePropertyChanged(); }
		}



		public SeriesCollection seriesCollection;
		public SeriesCollection SeriesCollection
		{
			get { return seriesCollection; }
			set { seriesCollection = value; RaisePropertyChanged(); }
		}

		public List<string> labels;
		public List<string> Labels
		{
			get { return labels; }
			set { labels = value; RaisePropertyChanged(); }
		}

		private readonly IRegionManager regionManager;

		public DelegateCommand<string> RegionTo { get; set; }
		public DelegateCommand<StudentInfo> SelectedCommand { get; set; }
		public DelegateCommand<DetailHomeWorkInfo> HomeWorkSelectedCommand { get; set; }
		public DelegateCommand<DetailHomeWorkSetInfo> HomeWorkSetSelectedCommand { get; set; }
		public DelegateCommand DeleteQuestionCommand { get; set; }
		public DelegateCommand UpdateQuestionCommand { get; set; }
		public DelegateCommand<MouseButtonEventArgs> ShowContextMenuCommand { get; set; }
		public DelegateCommand<QuestionData> RemoveNewColumnCommand { get; set; }
		public DelegateCommand<QuestionData> SelectedQuestionChangedCommand { get; set; }
		public DelegateCommand RegionToDetailCommand { get; set; }
		public DelegateCommand UpdateHomeWorkCommand { get; set; }
		public DelegateCommand DeleteHomeworkCommand { get; set; }
		public DelegateCommand AddNewColumnCommand { get; set; }
		public DelegateCommand SubmitAddQuestionsCommand { get; set; }
		public DelegateCommand ClearnAddQuestionsCommand { get; set; }
		public DelegateCommand SaveAllCommand { get; set; }
		public DelegateCommand PublicHomeWorkCommand { get; set; }
		public DelegateCommand FreshAllCommand { get; set; }

		Students(IRegionManager regionManager)
		{
			CreateMenuBar();

			RegionTo = new DelegateCommand<string>((x) =>
			{
				regionManager.Regions[PrismManager.MainRegionName].RequestNavigate(x.ToString());
				if (x == "DetailCheckView")
				{
					RegionToDetailCheckView();
				}
				if (x == "DetailView")
				{
					RegionToDetailView();
				}
				RegionToStudents();
			});

			UpdateHomeWorkCommand = new DelegateCommand(() => UpdateHomeWork());

			SelectedCommand = new DelegateCommand<StudentInfo>((x) =>
			{
				if (x == null) return;
				SelectedStudent = x;

				UpdateGird();
				RegionToDetailView();
			});

			HomeWorkSetSelectedCommand = new DelegateCommand<DetailHomeWorkSetInfo>((x) =>
			{
				if (x == null || SelectedStudent == null) return;
				SelectedHomeWorkSet = x;
				HomeWork = SelectedStudent.HomeWorkSet.GetDetailHomeWorkList(SelectedHomeWorkSet.Lesson);
			});

			HomeWorkSelectedCommand = new DelegateCommand<DetailHomeWorkInfo>((e) =>
			{
				if (e == null) return;
				//selectedHomeWork = e;
				e.Status = true;
			});

			DeleteQuestionCommand = new DelegateCommand(() =>
			{
				if (SelectedQuestion == null) return;
				var c = MessageBox.Show("确定删除吗？", "确认", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				QuestionLib.Remove(SelectedQuestion);
				QuestionDatas = QuestionLib.GetAllQuestion();
			});

			UpdateQuestionCommand = new DelegateCommand(() =>
			{
				if (SelectedQuestion == null) return;
				var c = MessageBox.Show("确定修改吗？", "确认", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				QuestionLib.Update(SelectedQuestion);
				SelectedQuestion.UpdateTime = DateTime.Now;
				SQLHelper.Instance.UpdateQuestion(SelectedQuestion);
			});

			ShowContextMenuCommand = new DelegateCommand<MouseButtonEventArgs>((e) =>
			{
				if (e.OriginalSource is FrameworkElement element)
				{
					var contextMenu = element.FindResource("MyContextMenu") as ContextMenu;
					if (contextMenu != null)
					{
						contextMenu.PlacementTarget = element;
						contextMenu.Placement = System.Windows.Controls.Primitives.PlacementMode.MousePoint;
						contextMenu.IsOpen = true;
					}
				}
			});

			SelectedQuestionChangedCommand = new DelegateCommand<QuestionData>((x) =>
			{
				if (x == null) return;
				SelectedQuestion = x;

			});

			RemoveNewColumnCommand = new DelegateCommand<QuestionData>((x) =>
			{
				if (x == null) return;
				var c = MessageBox.Show("确定删除吗？", "确认", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				ADDQuestionDatas.Remove(x);
			});

			AddNewColumnCommand = new DelegateCommand(() =>
			{
				ADDQuestionDatas.Add(new QuestionData { });
			});

			ClearnAddQuestionsCommand = new DelegateCommand(() =>
			{
				var c = MessageBox.Show("确定清除吗？", "清除所有题目", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				ADDQuestionDatas.Clear();
			});

			SaveAllCommand = new DelegateCommand(() => FileSystem.Instance.SaveAll());
			FreshAllCommand = new DelegateCommand(() =>
			{
				FileSystem.Instance.FreashAllInfo();
				Reload();
			});
			PublicHomeWorkCommand = new DelegateCommand(() =>
			{
				HomeWorkManager.Instance.PublicHomework(PublicLesson, IsAddPublicQuestionsLib, isNeedErrorset);
			});


			DeleteHomeworkCommand = new DelegateCommand(() =>
			{

				var cq = new CursonQuestionsData { Lesson = SelectedHomeWorkSet.Lesson, UID = SelectedStudent.UID };
				
				if(!SQLHelper.Instance.Delete(cq))
				{
					//TODO: NOTIFY:
					return;
				}

				if (SelectedHomeWorkSet != null && SelectedStudent != null)
					SelectedStudent.HomeWorkSet.DeleteHomework(SelectedHomeWorkSet.Lesson);

				if (SelectedStudent != null)
				{
					HomeWorkSet = SelectedStudent.HomeWorkSet.GetDetailHomeWorkSetList();
					SelectedStudent.HomeWorkSet.UpdateDate();
				}



			});


			SubmitAddQuestionsCommand = new DelegateCommand(() =>
			{

				Debug.Assert(Lesson != 0);

				foreach (var item in ADDQuestionDatas)
				{
					item.Id = ConvertToHashInt(item.Stem);
					item.Lesson = Lesson;
					if (!(QuestionLib.Get(item.Id) == null)) continue;
					if (string.IsNullOrEmpty(item.Stem) || string.IsNullOrEmpty(item.Source) || string.IsNullOrEmpty(item.Answer)) continue;
					QuestionLib.Add(item);
					preToRemove.Add(item);
				}

				foreach (var item in preToRemove)
				{
					ADDQuestionDatas.Remove(item);
				}

				QuestionLib.Save();
			});


			Load();

			StudentDatas = new ObservableCollection<StudentInfo>(studentDatas);
			QuestionDatas = new ObservableCollection<QuestionData>(questionDatas);
			ErrorSetDatas = new ObservableCollection<DetailErrorInfo>(errorSetDatas);
			MenuBars = new ReadOnlyObservableCollection<MenuBar>(menuBars);
			HomeWorkSet = new ObservableCollection<DetailHomeWorkSetInfo>(homeWorkSet);


			UpdateGird();

			this.regionManager = regionManager;

		}

		private void UpdateGird()
		{
			if(SelectedStudent == null) return;

			SelectedStudent.HomeWorkSet.UpdateDataView();

			SeriesCollection = new SeriesCollection
			{
				new LineSeries
				{
					Title = "错题数量",
					Values = SelectedStudent.HomeWorkSet.ErrorCounts,
				},
				new LineSeries
				{
					Title = "正确数量",
					Values = SelectedStudent.HomeWorkSet.CorrectCounts,
				}
			};

			Labels = SelectedStudent.HomeWorkSet.Lessons;
		}
	

		private int ConvertToHashInt(string item)
		{
			var hash = SHA256.Create();
			var result = hash.ComputeHash(Encoding.UTF8.GetBytes(item));
			return BitConverter.ToInt32(result, 0);
		}



		private void UpdateHomeWork()
		{
			// update error set and local homework
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Questions.ForEach(x =>
			{
				if (x.Status != HomeWork.FirstOrDefault(hw => hw.PID == x.ID).Status || selectedHomeWorkSet.Status == false)
				{
					x.Status = HomeWork.FirstOrDefault(hw => hw.PID == x.ID).Status;
					SelectedStudent.ErrorSet.AddQuestion(x);
					ErrorBase error = SelectedStudent.ErrorSet.Get(x.ID) ?? new ErrorBase {  QuestionBase = new QuestionBase { ID = x.ID, Status = x.Status} };
					SQLHelper.Instance.UpdateErrorSet(SelectedStudent.UID, error);
				}

			});
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).UpdateData();
			SelectedStudent.HomeWorkSet.UpdateDate();
			SQLHelper.Instance.UpdateErrorSetUserInfo(SelectedStudent.UID, (uint)SelectedStudent.ErrorSet.TotalCount, (uint)SelectedStudent.ErrorSet.CorrectCount);
			// update homework other info and online info
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Status = true;
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).DateTime = DateTime.Now;
			List<int> correctHomeworkArray = new List<int>();
			List<int> totalHomeworkArray = new List<int>();
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Questions.ForEach(x =>
			{
				totalHomeworkArray.Add(x.ID);
				if (x.Status != false) correctHomeworkArray.Add(x.ID);

			});
			SQLHelper.Instance.UpdateHomework(SelectedStudent.UID, SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Lesson,
				JsonConvert.SerializeObject(totalHomeworkArray),
				JsonConvert.SerializeObject(correctHomeworkArray), SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).DateTime,
				SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Status);



			//TODO: Notify
		}

		private void UpdateData()
		{

		}

		private void RegionToDetailView()
		{
			if (selectedStudent == null) return;
			ErrorSetDatas = selectedStudent.ErrorSet.GetDetailErrorQuestionList();
			HomeWorkSet = selectedStudent.HomeWorkSet.GetDetailHomeWorkSetList();
		}

		private void RegionToDetailCheckView()
		{
			if (selectedHomeWorkSet == null) return;
			HomeWork = SelectedStudent.HomeWorkSet.GetDetailHomeWorkList(SelectedHomeWorkSet.Lesson);
		}

		private void RegionToStudents()
		{

		}

		void CreateMenuBar()
		{
			menuBars.Add(new MenuBar() { Icon = "ForumPlusOutline", Title = "学生", NameSpace = "StudentsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "详情", NameSpace = "DetailView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "题库", NameSpace = "QuestionsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "审批", NameSpace = "CheckView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "发布家庭作业", NameSpace = "AddHomeWork" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "出题", NameSpace = "AddQuestionView" });
		}

		private void Load()
		{
			StudentLib.Load();
			QuestionLib.Load();
			Reload();
		}

		private void Reload()
		{
			studentDatas.Clear();
			questionDatas.Clear();
			StudentDatas = StudentLib.GetAllStudents();
			QuestionDatas = QuestionLib.GetAllQuestion();
		}


		private void OnPublicHomeworkSelectionChanged()
		{
			HomeworkTestData.Clear();
			if (IsAddPublicQuestionsLib)
			{
				HomeworkTestData = QuestionLib.GetAllQuestionByLesson(PublicLesson);
			}
			if (IsNeedErrorset)
			{
				if (SelectedStudent != null)
				{
					foreach (var item in SelectedStudent.ErrorSet.GetDetailErrorQuestionList())
					{
						homeworkTestData.Add(item.QuestionData);
					}
				}

			}
		}

	}
}
